@@ -1,7 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /* e-cal-backend-file-factory.h
  *
- * Copyright (C) 2004  Novell, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -30,7 +30,7 @@ G_BEGIN_DECLS
 
 void                 eds_module_initialize (GTypeModule *module);
 void                 eds_module_shutdown   (void);
-void                 eds_module_list_types (const GType **types, int *num_types);
+void                 eds_module_list_types (const GType **types, gint *num_types);
 
 G_END_DECLS
 
