@@ -60,7 +60,7 @@ static CamelObjectClass *parent_class = NULL;
  * This creates a new CamelCipherContext object which is used to sign,
  * verify, encrypt and decrypt streams.
  *
- * Return value: the new CamelCipherContext
+ * Returns: the new CamelCipherContext
  **/
 CamelCipherContext *
 camel_cipher_context_new (CamelSession *session)
@@ -115,7 +115,7 @@ cipher_sign (CamelCipherContext *ctx, const gchar *userid, CamelCipherHash hash,
  * Converts the (unsigned) part @ipart into a new self-contained mime part @opart.
  * This may be a multipart/signed part, or a simple part for enveloped types.
  *
- * Return value: 0 for success or -1 for failure.
+ * Returns: 0 for success or -1 for failure.
  **/
 gint
 camel_cipher_sign (CamelCipherContext *context, const gchar *userid, CamelCipherHash hash,
@@ -157,7 +157,7 @@ cipher_verify (CamelCipherContext *context, struct _CamelMimePart *sigpart, Came
  * @sigstream is assumed to be the signature stream and is used to
  * verify the integirity of the @istream.
  *
- * Return value: a CamelCipherValidity structure containing information
+ * Returns: a CamelCipherValidity structure containing information
  * about the integrity of the input stream or %NULL on failure to
  * execute at all.
  **/
@@ -202,7 +202,7 @@ cipher_encrypt (CamelCipherContext *context, const gchar *userid, GPtrArray *rec
  * Encrypts (and optionally signs) the cleartext input stream and
  * writes the resulting ciphertext to the output stream.
  *
- * Return value: 0 for success or -1 for failure.
+ * Returns: 0 for success or -1 for failure.
  **/
 gint
 camel_cipher_encrypt (CamelCipherContext *context, const gchar *userid, GPtrArray *recipients,
@@ -242,7 +242,7 @@ cipher_decrypt(CamelCipherContext *context, struct _CamelMimePart *ipart, struct
  *
  * Decrypts @ipart into @opart.
  *
- * Return value: A validity/encryption status.
+ * Returns: A validity/encryption status.
  **/
 CamelCipherValidity *
 camel_cipher_decrypt(CamelCipherContext *context, struct _CamelMimePart *ipart, struct _CamelMimePart *opart, CamelException *ex)
@@ -361,6 +361,10 @@ ccv_certinfo_free(CamelCipherCertInfo *info)
 {
 	g_free(info->name);
 	g_free(info->email);
+
+	if (info->cert_data && info->cert_data_free)
+		info->cert_data_free (info->cert_data);
+
 	g_free(info);
 }
 
@@ -444,13 +448,19 @@ camel_cipher_validity_clone(CamelCipherValidity *vin)
 
 	info = (CamelCipherCertInfo *)vin->sign.signers.head;
 	while (info->next) {
-		camel_cipher_validity_add_certinfo(vo, CAMEL_CIPHER_VALIDITY_SIGN, info->name, info->email);
+		if (info->cert_data && info->cert_data_clone && info->cert_data_free)
+			camel_cipher_validity_add_certinfo_ex (vo, CAMEL_CIPHER_VALIDITY_SIGN, info->name, info->email, info->cert_data_clone (info->cert_data), info->cert_data_free, info->cert_data_clone);
+		else
+			camel_cipher_validity_add_certinfo (vo, CAMEL_CIPHER_VALIDITY_SIGN, info->name, info->email);
 		info = info->next;
 	}
 
 	info = (CamelCipherCertInfo *)vin->encrypt.encrypters.head;
 	while (info->next) {
-		camel_cipher_validity_add_certinfo(vo, CAMEL_CIPHER_VALIDITY_ENCRYPT, info->name, info->email);
+		if (info->cert_data && info->cert_data_clone && info->cert_data_free)
+			camel_cipher_validity_add_certinfo_ex (vo, CAMEL_CIPHER_VALIDITY_SIGN, info->name, info->email, info->cert_data_clone (info->cert_data), info->cert_data_free, info->cert_data_clone);
+		else
+			camel_cipher_validity_add_certinfo (vo, CAMEL_CIPHER_VALIDITY_ENCRYPT, info->name, info->email);
 		info = info->next;
 	}
 
@@ -468,6 +478,19 @@ camel_cipher_validity_clone(CamelCipherValidity *vin)
  **/
 void
 camel_cipher_validity_add_certinfo(CamelCipherValidity *vin, enum _camel_cipher_validity_mode_t mode, const gchar *name, const gchar *email)
+{
+	camel_cipher_validity_add_certinfo_ex (vin, mode, name, email, NULL, NULL, NULL);
+}
+
+/**
+ * camel_cipher_validity_add_certinfo_ex:
+ *
+ * Add a cert info to the signer or encrypter info, with extended data set.
+ *
+ * Since: 2.30
+ **/
+void
+camel_cipher_validity_add_certinfo_ex (CamelCipherValidity *vin, camel_cipher_validity_mode_t mode, const gchar *name, const gchar *email, gpointer cert_data, void (*cert_data_free)(gpointer cert_data), gpointer (*cert_data_clone)(gpointer cert_data))
 {
 	CamelCipherCertInfo *info;
 	CamelDList *list;
@@ -475,11 +498,21 @@ camel_cipher_validity_add_certinfo(CamelCipherValidity *vin, enum _camel_cipher_
 	info = g_malloc0(sizeof(*info));
 	info->name = g_strdup(name);
 	info->email = g_strdup(email);
+	if (cert_data) {
+		if (cert_data_free && cert_data_clone) {
+			info->cert_data = cert_data;
+			info->cert_data_free = cert_data_free;
+			info->cert_data_clone = cert_data_clone;
+		} else {
+			if (!cert_data_free)
+				g_warning ("%s: requires non-NULL cert_data_free function!", G_STRFUNC);
+			if (!cert_data_clone)
+				g_warning ("%s: requires non-NULL cert_data_clone function!", G_STRFUNC);
+		}
+	}
 
 	list = (mode==CAMEL_CIPHER_VALIDITY_SIGN)?&vin->sign.signers:&vin->encrypt.encrypters;
 	camel_dlist_addtail(list, (CamelDListNode *)info);
-
-	printf("adding certinfo %s <%s>\n", name?name:"unset", email?email:"unset");
 }
 
 /**
@@ -641,7 +674,7 @@ cc_prepare_sign(CamelMimePart *part)
  *
  * The transfer encoding paramaters for the part may be changed by this function.
  *
- * Return value: -1 on error;
+ * Returns: -1 on error;
  **/
 gint
 camel_cipher_canonical_to_stream(CamelMimePart *part, guint32 flags, CamelStream *ostream)
