@@ -24,6 +24,7 @@
 
 #include <camel/camel-msgport.h>
 #include <camel/camel-list-utils.h>
+#include <libedataserver/e-flag.h>
 
 struct _CamelFolder;
 struct _CamelException;
@@ -50,19 +51,22 @@ struct _CamelIMAPXServer {
 	struct _CamelURL *url;
 	struct _CamelIMAPXStream *stream;
 	struct _capability_info *cinfo;
+	gboolean is_ssl_stream;
 
 	CamelIMAPXNamespaceList *nsl;
 
 	/* incoming jobs */
 	CamelMsgPort *port;
 	CamelDList jobs;
+	/* in micro seconds */
+	guint job_timeout;
 
 	gchar tagprefix;
 	gint state:4;
 
 	/* Current command/work queue.  All commands are stored in one list,
 	   all the time, so they can be cleaned up in exception cases */
-	gpointer queue_lock;
+	GStaticRecMutex queue_lock;
 	struct _CamelIMAPXCommand *literal;
 	CamelDList queue;
 	CamelDList active;
@@ -79,12 +83,26 @@ struct _CamelIMAPXServer {
 	guint32 exists;
 	guint32 recent;
 	guint32 mode;
+	guint32 unread;
 
 	/* any expunges that happened from the last command, they are
 	   processed after the command completes. */
-	GArray *expunged;
-	
-	GMutex *connect_lock;
+	GSList *expunged;
+
+	GThread *parser_thread;
+	/* Protects the output stream between parser thread (which can disconnect from server) and other threads that issue
+	   commands. Input stream does not require a lock since only parser_thread can operate on it */
+	GStaticRecMutex ostream_lock;
+	/* Used for canceling operations as well as signaling parser thread to disconnnect/quit */
+	CamelOperation *op;
+	gboolean parser_quit;
+
+	/* Idle */
+	struct _CamelIMAPXIdle *idle;
+	gboolean use_idle;
+
+	/* used for storing eflags to syncronize duplicate get_message requests */
+	GHashTable *uid_eflags;
 };
 
 struct _CamelIMAPXServerClass {
@@ -96,15 +114,23 @@ struct _CamelIMAPXServerClass {
 CamelType               camel_imapx_server_get_type     (void);
 CamelIMAPXServer *camel_imapx_server_new(struct _CamelStore *store, struct _CamelURL *url);
 
-gboolean camel_imapx_server_connect(CamelIMAPXServer *is, gint state);
+gboolean camel_imapx_server_connect(CamelIMAPXServer *is, gint state, CamelException *ex);
+gboolean imapx_connect_to_server (CamelIMAPXServer *is, CamelException *ex);
 
 GPtrArray *camel_imapx_server_list(CamelIMAPXServer *is, const gchar *top, guint32 flags, CamelException *ex);
 
 void camel_imapx_server_refresh_info(CamelIMAPXServer *is, CamelFolder *folder, struct _CamelException *ex);
-void camel_imapx_server_sync_changes(CamelIMAPXServer *is, CamelFolder *folder, GPtrArray *infos, CamelException *ex);
+void camel_imapx_server_sync_changes(CamelIMAPXServer *is, CamelFolder *folder, CamelException *ex);
 void camel_imapx_server_expunge(CamelIMAPXServer *is, CamelFolder *folder, CamelException *ex);
+void camel_imapx_server_noop (CamelIMAPXServer *is, CamelFolder *folder, CamelException *ex);
 
 CamelStream *camel_imapx_server_get_message(CamelIMAPXServer *is, CamelFolder *folder, const gchar *uid, struct _CamelException *ex);
+void camel_imapx_server_copy_message (CamelIMAPXServer *is, CamelFolder *source, CamelFolder *dest, GPtrArray *uids, gboolean delete_originals, CamelException *ex);
 void camel_imapx_server_append_message(CamelIMAPXServer *is, CamelFolder *folder, struct _CamelMimeMessage *message, const struct _CamelMessageInfo *mi, CamelException *ex);
+void camel_imapx_server_sync_message (CamelIMAPXServer *is, CamelFolder *folder, const gchar *uid, CamelException *ex);
 
-#endif /* ! _CAMEL_IMAPX_SERVER_H */
+void camel_imapx_server_manage_subscription (CamelIMAPXServer *is, const gchar *folder_name, gboolean subscribe, CamelException *ex);
+void camel_imapx_server_create_folder (CamelIMAPXServer *is, const gchar *folder_name, CamelException *ex);
+void camel_imapx_server_delete_folder (CamelIMAPXServer *is, const gchar *folder_name, CamelException *ex);
+
+#endif /* _CAMEL_IMAPX_SERVER_H */
