@@ -129,6 +129,22 @@ primary_selection_changed_cb (ESourceSelector *selector, gpointer user_data)
 	if (priv->selected_source)
 		g_object_unref (priv->selected_source);
 	priv->selected_source = e_source_selector_peek_primary_selection (selector);
+
+	/* FIXME: add an API to "except-source" or to get the ESourceSelector from outside */
+	if (priv->selected_source) {
+		ESource *except_source = g_object_get_data (G_OBJECT (dialog), "except-source");
+
+		if (except_source) {
+			const gchar *except_uid, *selected_uid;
+
+			except_uid = e_source_peek_uid (except_source);
+			selected_uid = e_source_peek_uid (priv->selected_source);
+
+			if (except_uid && selected_uid && g_str_equal (except_uid, selected_uid))
+				priv->selected_source = NULL;
+		}
+	}
+
 	if (priv->selected_source) {
 		g_object_ref (priv->selected_source);
 		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK, TRUE);
@@ -141,7 +157,7 @@ setup_dialog (GtkWindow *parent, ESourceSelectorDialog *dialog, ESourceList *sou
 {
 	GtkWidget *vbox, *label, *scroll, *hbox, *spacer;
 	GtkWidget *content_area;
-	char *label_text;
+	gchar *label_text;
 	ESourceSelectorDialogPrivate *priv = dialog->priv;
 
 	priv->source_list = g_object_ref (source_list);
@@ -198,7 +214,7 @@ setup_dialog (GtkWindow *parent, ESourceSelectorDialog *dialog, ESourceList *sou
  *
  * Create a new source selector dialog for the given @list.
  *
- * Return value: The newly created widget.
+ * Returns: The newly created widget.
  */
 GtkWidget *
 e_source_selector_dialog_new (GtkWindow *parent, ESourceList *source_list)
@@ -212,13 +228,61 @@ e_source_selector_dialog_new (GtkWindow *parent, ESourceList *source_list)
 	return setup_dialog (parent, dialog, source_list);
 }
 
+/**
+ * e_source_selector_dialog_select_default_source:
+ * @dialog: An #ESourceSelectorDialog widget.
+ *
+ * Selects default source in the dialog.
+ *
+ * Returns: Whether found any default source.
+ *
+ * Since: 2.28
+ **/
+gboolean
+e_source_selector_dialog_select_default_source (ESourceSelectorDialog *dialog)
+{
+	ESourceSelectorDialogPrivate *priv;
+
+	g_return_val_if_fail (E_IS_SOURCE_SELECTOR_DIALOG (dialog), FALSE);
+
+	priv = dialog->priv;
+
+	if (priv->source_list) {
+		ESource *default_source = NULL;
+		GSList *groups, *g;
+
+		groups = e_source_list_peek_groups (priv->source_list);
+		for (g = groups; g != NULL && !default_source; g = g->next) {
+			ESourceGroup *group = E_SOURCE_GROUP (g->data);
+			GSList *sources, *s;
+
+			sources = e_source_group_peek_sources (group);
+
+			for (s = sources; s != NULL && !default_source; s = s->next) {
+				ESource *source = E_SOURCE (s->data);
+
+				if (source && e_source_get_property (source, "default"))
+					default_source = source;
+			}
+
+		}
+
+		if (default_source)
+			e_source_selector_set_primary_selection (E_SOURCE_SELECTOR (priv->source_selector), default_source);
+
+		return default_source != NULL;
+	}
+
+	return FALSE;
+}
+
 /**
  * e_source_selector_dialog_peek_primary_selection:
  * @dialog: An #ESourceSelectorDialog widget.
  *
  * Peek the currently selected source in the given @dialog.
  *
- * Return value: the selected ESource.
+ * Returns: the selected ESource.
  */
 ESource *
 e_source_selector_dialog_peek_primary_selection (ESourceSelectorDialog *dialog)
